varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;

uniform sampler2D u_texture;
uniform vec4 u_HSV;

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-4;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main()
{
    //vec4 textureColor = v_color * texture2D(u_texture, v_texCoord);
    vec4 texColor = texture2D(u_texture, v_texCoord);

    float e = 1.0e-4;
    texColor.rgb /= texColor.a+e;
    
    vec4 textureColor = v_color * texColor;

    vec3 fragRGB = textureColor.rgb * textureColor.a;
    vec3 fragHSV = rgb2hsv(fragRGB).xyz;
    fragHSV.x += u_HSV.x;
    fragHSV.yz *= u_HSV.yz;
    fragRGB = hsv2rgb(fragHSV);
    float a = textureColor.a * u_HSV.a;
    gl_FragColor = vec4(fragRGB, a);
}
