varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;

uniform sampler2D u_texture;

// Each spotlight has 3 dimensions:
// x & y represent the absolute position of the center of the spotlight
// z represents the radius of the spotlight
// w is ignored

// We are going to have a maximum of 5 "spotlighted" leves + stella

uniform vec4 u_spotlightLevel01;
uniform vec4 u_spotlightLevel02;
uniform vec4 u_spotlightLevel03;
uniform vec4 u_spotlightLevel04;
uniform vec4 u_spotlightLevel05;

uniform vec4 u_spotlightStella;
uniform vec4 u_spotlightChest;

// using a vec4 since it seems that using a float gives problems in Fiction Editor
uniform vec4 u_darkLayerAspectRatio;

const vec4 spotlightColor = vec4(0, 0, 0, 0);

//----------------------------------------------------------------------------------------------------------------------
float getCircleDistance(vec2 position, vec3 spotlight)
{
    return length(position - spotlight.xy) - spotlight.z;
}

//----------------------------------------------------------------------------------------------------------------------
float getSpotlightInfluenceForPosition(in vec4 spotlight, in vec2 position, in float currentAlpha)
{
    float distance = getCircleDistance(position, spotlight.xyz);
    float alphaInterpolator = smoothstep(0.0, 0.7, (abs(distance) / spotlight.z));
    float spotlightAlpha = mix(currentAlpha, spotlightColor.a, alphaInterpolator);

    return mix(spotlightAlpha, currentAlpha, step(0.0, distance));
}

//----------------------------------------------------------------------------------------------------------------------
void main()
{
    // Apply aspect ratio transformation to the input coordinates
    vec2 currentPosition = v_texCoord;
    currentPosition.y *= u_darkLayerAspectRatio.x;

    // Compose the "ilumination" for the current position
    vec4 color = v_color;

    color.a = getSpotlightInfluenceForPosition(u_spotlightLevel01, currentPosition, color.a);
    color.a = getSpotlightInfluenceForPosition(u_spotlightLevel02, currentPosition, color.a);
    color.a = getSpotlightInfluenceForPosition(u_spotlightLevel03, currentPosition, color.a);
    color.a = getSpotlightInfluenceForPosition(u_spotlightLevel04, currentPosition, color.a);
    color.a = getSpotlightInfluenceForPosition(u_spotlightLevel05, currentPosition, color.a);
	 color.a = getSpotlightInfluenceForPosition(u_spotlightChest, currentPosition, color.a);
    color.a = getSpotlightInfluenceForPosition(u_spotlightStella, currentPosition, color.a);

    gl_FragColor = vec4(v_color.rgb, color.a);
}
